=== modified file 'lib/canonical/codehosting/puller/worker.py'
--- lib/canonical/codehosting/puller/worker.py	2007-10-05 05:11:59 +0000
+++ lib/canonical/codehosting/puller/worker.py	2007-10-11 23:38:56 +0000
@@ -1,4 +1,4 @@
-# Copyright 2006 Canonical Ltd.  All rights reserved.
+# Copyright 2006-2007 Canonical Ltd.  All rights reserved.
 
 __metaclass__ = type
 
@@ -28,7 +28,7 @@
     'BranchReferenceLoopError',
     'BranchReferenceForbidden',
     'BranchReferenceValueError',
-    'get_canonical_url',
+    'get_canonical_url_for_branch_name',
     'PullerWorker',
     'PullerWorkerProtocol'
     ]
@@ -68,10 +68,10 @@
     """
 
 
-def get_canonical_url(unique_name):
+def get_canonical_url_for_branch_name(unique_name):
     """Custom implementation of canonical_url(branch) for error reporting.
 
-    The actual canonical_url method cannot be used because we do not have
+    The actual `canonical_url` function cannot be used because we do not have
     access to real content objects.
     """
     if config.launchpad.vhosts.use_https:
@@ -83,31 +83,17 @@
 
 
 class PullerWorkerProtocol:
-
-    def __init__(self, output, error):
-        self.output = output
-        self.error = error
+    """The protocol used to communicate with the puller scheduler.
+
+    This protocol notifies the scheduler of events such as startMirroring,
+    mirrorSucceeded and mirrorFailed.
+    """
+
+    def __init__(self, output):
+        self.out_stream = output
 
     def sendNetstring(self, string):
-        self.output.write('%d:%s,' % (len(string), string))
-
-    def _record_oops(self, branch_to_mirror, message=None):
-        """Record an oops for the current exception.
-
-        This must only be called while handling an exception.
-
-        :param message: custom explanatory error message. Do not use
-            str(exception) to fill in this parameter, it should only be set
-            when a human readable error has been explicitely generated.
-        """
-        request = errorlog.ScriptRequest([
-            ('branch_id', branch_to_mirror.branch_id),
-            ('source', branch_to_mirror.source),
-            ('dest', branch_to_mirror.dest),
-            ('error-explanation', str(message))])
-        request.URL = get_canonical_url(branch_to_mirror.unique_name)
-        errorlog.globalErrorUtility.raising(sys.exc_info(), request)
-        return request.oopsid
+        self.out_stream.write('%d:%s,' % (len(string), string))
 
     def startMirroring(self, branch_to_mirror):
         self.sendNetstring('startMirroring')
@@ -116,10 +102,10 @@
         self.sendNetstring('mirrorSucceeded')
         self.sendNetstring(str(last_revision))
 
-    def mirrorFailed(self, branch_to_mirror, message):
+    def mirrorFailed(self, branch_to_mirror, message, oops_id):
         self.sendNetstring('mirrorFailed')
         self.sendNetstring(str(message))
-        self.sendNetstring(self._record_oops(branch_to_mirror, message))
+        self.sendNetstring(oops_id)
 
 
 def identical_formats(branch_one, branch_two):
@@ -282,6 +268,26 @@
         branch.pull(self._source_branch)
         return branch
 
+    def _record_oops(self, message=None):
+        """Record an oops for the current exception.
+
+        This must only be called while handling an exception.
+
+        :param message: custom explanatory error message. Do not use
+            str(exception) to fill in this parameter, it should only be set
+            when a human readable error has been explicitly generated.
+        """
+        request = errorlog.ScriptRequest([
+            ('branch_id', self.branch_id), ('source', self.source),
+            ('dest', self.dest), ('error-explanation', str(message))])
+        request.URL = get_canonical_url_for_branch_name(self.unique_name)
+        errorlog.globalErrorUtility.raising(sys.exc_info(), request)
+        return request.oopsid
+
+    def _mirrorFailed(self, error):
+        oops_id = self._record_oops(error)
+        self.protocol.mirrorFailed(self, error, oops_id)
+
     def mirror(self):
         """Open source and destination branches and pull source into
         destination.
@@ -302,28 +308,28 @@
                 # be able to get rid of this.
                 # https://launchpad.net/products/bzr/+bug/42383
                 msg = "Authentication required."
-            self.protocol.mirrorFailed(self, msg)
+            self._mirrorFailed(msg)
 
         except socket.error, e:
             msg = 'A socket error occurred: %s' % str(e)
-            self.protocol.mirrorFailed(self, msg)
+            self._mirrorFailed(msg)
 
         except UnsupportedFormatError, e:
             msg = ("Launchpad does not support branches from before "
                    "bzr 0.7. Please upgrade the branch using bzr upgrade.")
-            self.protocol.mirrorFailed(self, msg)
+            self._mirrorFailed(msg)
 
         except UnknownFormatError, e:
-            self.protocol.mirrorFailed(self, e)
+            self._mirrorFailed(e)
 
         except (ParamikoNotPresent, BadUrlSsh), e:
             msg = ("Launchpad cannot mirror branches from SFTP and SSH URLs."
                    " Please register a HTTP location for this branch.")
-            self.protocol.mirrorFailed(self, msg)
+            self._mirrorFailed(msg)
 
         except BadUrlLaunchpad:
             msg = "Launchpad does not mirror branches from Launchpad."
-            self.protocol.mirrorFailed(self, msg)
+            self._mirrorFailed(msg)
 
         except NotBranchError, e:
             hosted_branch_error = NotBranchError(
@@ -333,23 +339,23 @@
                 BranchType.IMPORTED: "Not a branch.",
                 }
             msg = message_by_type.get(self.branch_type, str(e))
-            self.protocol.mirrorFailed(self, msg)
+            self._mirrorFailed(msg)
 
         except BranchReferenceForbidden, e:
             msg = ("Branch references are not allowed for branches of type %s."
                    % (self.branch_type.title,))
-            self.protocol.mirrorFailed(self, msg)
+            self._mirrorFailed(msg)
 
         except BranchReferenceValueError, e:
             msg = "Bad branch reference value: %s" % (e.url,)
-            self.protocol.mirrorFailed(self, msg)
+            self._mirrorFailed(msg)
 
         except BranchReferenceLoopError, e:
             msg = "Circular branch reference."
-            self.protocol.mirrorFailed(self, msg)
+            self._mirrorFailed(msg)
 
         except BzrError, e:
-            self.protocol.mirrorFailed(self, e)
+            self._mirrorFailed(e)
 
         except (KeyboardInterrupt, SystemExit):
             # Do not record OOPS for those exceptions.

=== modified file 'lib/canonical/codehosting/tests/helpers.py'
--- lib/canonical/codehosting/tests/helpers.py	2007-10-09 23:23:30 +0000
+++ lib/canonical/codehosting/tests/helpers.py	2007-10-11 23:38:56 +0000
@@ -6,7 +6,8 @@
 __all__ = [
     'AvatarTestCase', 'CodeHostingTestProviderAdapter',
     'CodeHostingRepositoryTestProviderAdapter', 'FakeLaunchpad',
-    'ServerTestCase', 'adapt_suite', 'deferToThread']
+    'ServerTestCase', 'adapt_suite', 'create_branch_with_one_revision',
+    'deferToThread']
 
 import os
 import shutil
@@ -404,7 +405,7 @@
     return suite
 
 
-def create_branch(branch_dir):
+def create_branch_with_one_revision(branch_dir):
     os.makedirs(branch_dir)
     tree = bzrdir.BzrDir.create_standalone_workingtree(branch_dir)
     f = open(branch_dir + 'hello', 'w')

=== modified file 'lib/canonical/codehosting/tests/test_puller_scheduler.py'
--- lib/canonical/codehosting/tests/test_puller_scheduler.py	2007-10-08 20:44:55 +0000
+++ lib/canonical/codehosting/tests/test_puller_scheduler.py	2007-10-11 23:38:56 +0000
@@ -10,7 +10,8 @@
 from canonical.launchpad.interfaces import BranchType
 from canonical.codehosting import branch_id_to_path
 from canonical.codehosting.puller.branchtomirror import BranchToMirror
-from canonical.codehosting.tests.helpers import create_branch
+from canonical.codehosting.tests.helpers import (
+    create_branch_with_one_revision)
 from canonical.codehosting.puller import jobmanager
 from canonical.authserver.client.branchstatus import BranchStatusClient
 from canonical.authserver.tests.harness import AuthserverTacTestSetup
@@ -189,7 +190,7 @@
                                     branch
         """
         branch_dir = os.path.join(self.test_dir, relative_dir)
-        create_branch(branch_dir)
+        create_branch_with_one_revision(branch_dir)
         if target == None:
             target_dir = None
         else:

=== modified file 'lib/canonical/codehosting/tests/test_puller_worker.py'
--- lib/canonical/codehosting/tests/test_puller_worker.py	2007-10-10 21:17:16 +0000
+++ lib/canonical/codehosting/tests/test_puller_worker.py	2007-10-11 23:38:56 +0000
@@ -6,6 +6,7 @@
 
 import httplib
 import os
+import re
 import shutil
 import socket
 from StringIO import StringIO
@@ -16,6 +17,7 @@
 import bzrlib.branch
 from bzrlib import bzrdir
 from bzrlib.branch import BranchReferenceFormat
+from bzrlib.revision import ensure_null, NULL_REVISION
 from bzrlib.tests import TestCaseInTempDir, TestCaseWithTransport
 from bzrlib.tests.repository_implementations.test_repository import (
             TestCaseWithRepository)
@@ -28,9 +30,10 @@
 from canonical.codehosting import branch_id_to_path
 from canonical.codehosting.puller.worker import (
     PullerWorker, BadUrlSsh, BadUrlLaunchpad, BranchReferenceLoopError,
-    BranchReferenceForbidden, BranchReferenceValueError, get_canonical_url,
-    PullerWorkerProtocol)
-from canonical.codehosting.tests.helpers import create_branch
+    BranchReferenceForbidden, BranchReferenceValueError,
+    get_canonical_url_for_branch_name, PullerWorkerProtocol)
+from canonical.codehosting.tests.helpers import (
+    create_branch_with_one_revision)
 from canonical.launchpad.database import Branch
 from canonical.launchpad.interfaces import BranchType
 from canonical.launchpad.webapp import canonical_url
@@ -49,8 +52,9 @@
         self.calls.append(
             ('mirrorSucceeded', branch_to_mirror, last_revision))
 
-    def mirrorFailed(self, branch_to_mirror, message):
-        self.calls.append(('mirrorFailed', branch_to_mirror, message))
+    def mirrorFailed(self, branch_to_mirror, message, oops_id):
+        self.calls.append(
+            ('mirrorFailed', branch_to_mirror, message, oops_id))
 
 
 class StubbedPullerWorker(PullerWorker):
@@ -97,7 +101,7 @@
         if dest_dir is None:
             dest_dir = os.path.join(self.test_dir, 'dest_dir')
         if protocol is None:
-            protocol = PullerWorkerProtocol(StringIO(), StringIO())
+            protocol = PullerWorkerProtocol(StringIO())
         return PullerWorker(
             src_dir, dest_dir, branch_id=1, unique_name='foo/bar/baz',
             branch_type=branch_type, protocol=protocol)
@@ -170,7 +174,7 @@
     def testMirrorActuallyMirrors(self):
         # Check that mirror() will mirror the Bazaar branch.
         to_mirror = self.makePullerWorker()
-        tree = create_branch(to_mirror.source)
+        tree = create_branch_with_one_revision(to_mirror.source)
         to_mirror.mirror()
         mirrored_branch = bzrlib.branch.Branch.open(to_mirror.dest)
         self.assertEqual(
@@ -180,17 +184,16 @@
         # Check that we can mirror an empty branch, and that the
         # last_mirrored_id for an empty branch can be distinguished from an
         # unmirrored branch.
-
-        # Create a branch
         to_mirror = self.makePullerWorker()
 
-        # create empty source branch
+        # Create an empty source branch.
         os.makedirs(to_mirror.source)
-        tree = bzrdir.BzrDir.create_standalone_workingtree(to_mirror.source)
+        tree = bzrdir.BzrDir.create_branch_and_repo(to_mirror.source)
 
         to_mirror.mirror()
         mirrored_branch = bzrlib.branch.Branch.open(to_mirror.dest)
-        self.assertEqual(None, mirrored_branch.last_revision())
+        self.assertEqual(
+            NULL_REVISION, ensure_null(mirrored_branch.last_revision()))
 
 
 class TestPullerWorkerFormats(TestCaseWithRepository, PullerWorkerMixin):
@@ -290,6 +293,8 @@
     def assertMirrorFailed(self, puller_worker, message_substring):
         """Assert that puller_worker failed, and that message_substring is in
         the message.
+
+        'puller_worker' must use a StubbedPullerWorkerProtocol.
         """
         protocol = puller_worker.protocol
         self.assertEqual(
@@ -299,9 +304,8 @@
         startMirroring, mirrorFailed = protocol.calls
         self.assertEqual(('startMirroring', puller_worker), startMirroring)
         self.assertEqual(('mirrorFailed', puller_worker), mirrorFailed[:2])
-        self.assertTrue(
-            message_substring in str(mirrorFailed[2]),
-            "%r not in %r" % (message_substring, str(mirrorFailed[2])))
+        self.assertContainsRe(
+            str(mirrorFailed[2]), re.escape(message_substring))
 
     def testUnopenableSourceDoesNotCreateMirror(self):
         non_existent_source = os.path.abspath('nonsensedir')
@@ -755,15 +759,21 @@
         PullerWorkerMixin.setUp(self)
         self.test_dir = tempfile.mkdtemp()
         self.output = StringIO()
-        self.error = StringIO()
-        self.resetBuffers()
-        self.protocol = PullerWorkerProtocol(self.output, self.error)
+        self.protocol = PullerWorkerProtocol(self.output)
         self.branch_to_mirror = self.makePullerWorker()
 
     def tearDown(self):
         PullerWorkerMixin.tearDown(self)
 
+    def assertSentNetstrings(self, expected_netstrings):
+        """Assert that the protocol sent the given netstrings (in order)."""
+        observed_netstrings = self.getNetstrings(self.output.getvalue())
+        self.assertEqual(expected_netstrings, observed_netstrings)
+
     def getNetstrings(self, line):
+        """Return the sequence of strings that are the netstrings that make up
+        'line'.
+        """
         strings = []
         while len(line) > 0:
             colon_index = line.find(':')
@@ -773,48 +783,34 @@
             line = line[colon_index+length+2:]
         return strings
 
-    def assertNoError(self):
-        self.assertEqual('', self.error.getvalue())
-
     def resetBuffers(self):
         """Empty the test output and error buffers."""
         self.output.truncate(0)
-        self.error.truncate(0)
         self.assertEqual('', self.output.getvalue())
-        self.assertNoError()
 
     def test_nothingSentOnConstruction(self):
         """The protocol sends nothing until it receives an event."""
-        self.assertEqual('', self.output.getvalue())
-        self.assertNoError()
+        self.assertSentNetstrings([])
 
     def test_startMirror(self):
         """Calling startMirroring sends 'startMirroring' as a netstring."""
         self.protocol.startMirroring(self.branch_to_mirror)
-        [command] = self.getNetstrings(self.output.getvalue())
-        self.assertEqual('startMirroring', command)
-        self.assertNoError()
+        self.assertSentNetstrings(['startMirroring'])
 
     def test_mirrorSucceeded(self):
         """Calling 'mirrorSucceeded' sends the revno and 'mirrorSucceeded'."""
         self.protocol.startMirroring(self.branch_to_mirror)
         self.resetBuffers()
         self.protocol.mirrorSucceeded(self.branch_to_mirror, 1234)
-        [command, revno] = self.getNetstrings(self.output.getvalue())
-        self.assertEqual('mirrorSucceeded', command)
-        self.assertEqual('1234', revno)
-        self.assertNoError()
+        self.assertSentNetstrings(['mirrorSucceeded', '1234'])
 
     def test_mirrorFailed(self):
         """Calling 'mirrorFailed' sends the error message."""
         self.protocol.startMirroring(self.branch_to_mirror)
         self.resetBuffers()
-        self.protocol.mirrorFailed(self.branch_to_mirror, 'Error Message')
-        [command, message, oops] = self.getNetstrings(self.output.getvalue())
-        self.assertEqual('mirrorFailed', command)
-        self.assertEqual('Error Message', message)
-        self.failUnless(oops.startswith('OOPS'))
-        self.assertNoError()
+        self.protocol.mirrorFailed(
+            self.branch_to_mirror, 'Error Message', 'OOPS')
+        self.assertSentNetstrings(['mirrorFailed', 'Error Message', 'OOPS'])
 
 
 class TestCanonicalUrl(unittest.TestCase):
@@ -823,8 +819,8 @@
     layer = LaunchpadScriptLayer
 
     def testCanonicalUrlConsistent(self):
-        # worker.get_canonical_url is consistent with webapp.canonical_url, if
-        # the provided unique_name is correct.
+        # worker.get_canonical_url_for_branch_name is consistent with
+        # webapp.canonical_url, if the provided unique_name is correct.
         branch = Branch.get(15)
         # Check that the unique_name used in this test is consistent with the
         # sample data. This is an invariant of the test, so use a plain assert.
@@ -833,7 +829,8 @@
         # Now check that our implementation of canonical_url is consistent with
         # the canonical one.
         self.assertEqual(
-            canonical_url(branch), get_canonical_url(unique_name))
+            canonical_url(branch),
+            get_canonical_url_for_branch_name(unique_name))
 
 
 def test_suite():

=== modified file 'scripts/mirror-branch.py'
--- scripts/mirror-branch.py	2007-10-05 05:11:59 +0000
+++ scripts/mirror-branch.py	2007-10-11 23:38:56 +0000
@@ -1,6 +1,11 @@
 #!/usr/bin/python2.4
 # Copyright 2007 Canonical Ltd.  All rights reserved.
 
+# Script run by cronscripts/supermirror-pull.py to mirror individual branches.
+# This script does not use the standard Launchpad script framework as it is
+# not intended to be run by itself.
+
+
 import _pythonpath
 from optparse import OptionParser
 import sys
@@ -49,7 +54,7 @@
     shut_up_deprecation_warning()
     force_bzr_to_use_urllib()
 
-    protocol = PullerWorkerProtocol(sys.stdout, sys.stderr)
+    protocol = PullerWorkerProtocol(sys.stdout)
     PullerWorker(
         source_url, destination_url, int(branch_id), unique_name, branch_type,
         protocol).mirror()

